/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

#include <qstring.h>
#include <qapplication.h>
#include <qpopupmenu.h>
#include <qclipboard.h>
#include <qfiledialog.h>
#include <cstdlib>
#include <ctime>

using namespace std;

void password::init() // Initialize the program
{
    // Disable the Generate button because none of the
    // checkboxes have been checked yet
    // Using a constructor is certainly a much better approach!  ;-)
    btnGenerate->setEnabled(false);
   
}

void password::selectAll() // Selects all character modes
{
    optUpperCase->setChecked(true);
    optLowerCase->setChecked(true);
    optNumbers->setChecked(true);
    optSymbols->setChecked(true);
    btnGenerate->setEnabled(true);
}


void password::deselectAll() // Deselects all character modes
{
    optUpperCase->setChecked(false);
    optLowerCase->setChecked(false);
    optNumbers->setChecked(false);
    optSymbols->setChecked(false);
    btnGenerate->setEnabled(false);
}


void password::generate() // Generate the password
{    
    QString upperCase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    QString lowerCase = "abcdefghijklmnopqrstuvwxyz";
    QString numbers = "0123456789";
    QString symbols = "!\"$%&,()*+-./;:<=>?[\\]^_{|}~";
    
    // this is the variable that will contain what to be randomised.
    QString values; 
    
    if ( optUpperCase->isChecked() )
	values += upperCase;
    if ( optLowerCase->isChecked() )
	values += lowerCase;
    if ( optNumbers->isChecked() )
	values += numbers;
    if ( optSymbols->isChecked() )
	values += symbols;
    
    
    int size = values.length();
    
    // the keeper password.  this value is what's inserted into the list box
    QString password;        
    
    // First clear the contents before a new set is done, to prevent appending more values
    listPassword->clear();   
    
    // Clearly, the value of the spinner
    int numPasswords = numPassDrop->value();     
    
    // Length of the password to be generated
    int passwordLength = lenPassDrop->value();     
    
    // This will get a random index of the string to get a random password
    int rndInt;                                                          
    
    // Randomise the seed
    srand(static_cast<unsigned>(time(0)));     
    
    for ( int i = 0; i < numPasswords; ++i )
    {
	for ( int j = 0; j < passwordLength; ++j )
	{
	    
	    // 0 to the size of the string
	    rndInt = rand() % size;         
	    
	    // grab the char at element rndInt inside the string of values
	    password += values[rndInt];        
	}
	
	// Populate the listbox
	listPassword->insertItem(password, i);           
	
	// reset the string to a null value in order to create new passwords
	password = QString::null;                                
    }
}

void password::reset()
{
    
    // Firstly, let's untick all of the options
    optUpperCase->setChecked(false);
    optLowerCase->setChecked(false);
    optNumbers->setChecked(false);
    optSymbols->setChecked(false);
    
    // reset the spinner value for the numPass
    numPassDrop->setValue(1);
    
    // reset the spinner value for the lenPass
    lenPassDrop->setValue(6);
    
    // we now clear the password listbox
    listPassword->clear();
    
    // Lastly, we disbale the Generate button
    btnGenerate->setEnabled(false);
}

void password::disableGenerate()
{
    if ( optUpperCase->isChecked() || optLowerCase->isChecked() || 
	 optNumbers->isChecked() || optSymbols->isChecked() ) 
	btnGenerate->setEnabled(true);
    else
	btnGenerate->setEnabled(false);
}

void password::copy() 
{   
      // Puts the selected item in the list into the clipboard and the X11 mouse selection
	
	QClipboard *cb = QApplication::clipboard();
	cb->setText(listPassword->currentText(), QClipboard::Clipboard );       
	if ( cb->supportsSelection() )
		cb->setText(listPassword->currentText(), QClipboard::Selection );
}

void password::remove()
{
	listPassword->removeItem( listPassword->currentItem() );
}

void password::save()
{
	QString fn = QFileDialog::getSaveFileName( "", "Text Files (*.txt)", this );
	QFile f (fn );
	if ( f.open ( IO_WriteOnly ) ) 
	{
		QTextStream ts ( &f );
		for ( uint i = 0; i < listPassword->count(); ++i )
		{
			ts << listPassword->text(i) << "\n";
		}
	}
}

void password::contextMenuEvent( QContextMenuEvent * )
{
    QPopupMenu* contextMenu = new QPopupMenu( this );
    Q_CHECK_PTR( contextMenu );
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>Options</b></u></font>", this );
    caption->setAlignment( Qt::AlignCenter );
    contextMenu->insertItem( caption );
  
    int copyID = contextMenu->insertItem( "&Copy",  this, SLOT(copy()) );
    int removeID = contextMenu->insertItem( "&Remove", this, SLOT(remove()) );	 
    int saveID = contextMenu->insertItem( "&Save", this, SLOT(save()) );
	  
    if ( listPassword->currentItem() == -1 )
   {
		 contextMenu->setItemEnabled(saveID, FALSE);	 
		 contextMenu->setItemEnabled(copyID, FALSE);
		 contextMenu->setItemEnabled(removeID, FALSE);
   }
    contextMenu->exec( QCursor::pos() );
    delete contextMenu;
}
